@@ -748,7 +748,7 @@ plot.soap.film <- function(x,P=NULL,data=NULL,label="",se1.mult=1,se2.mult=2,
                      partial.resids=FALSE,rug=TRUE,se=TRUE,scale=-1,n=100,n2=40,
                      pers=FALSE,theta=30,phi=30,jit=FALSE,xlab=NULL,ylab=NULL,main=NULL,
                      ylim=NULL,xlim=NULL,too.far=0.1,shade=FALSE,shade.col="gray80",
-                     shift=0,trans=I,by.resids=FALSE,scheme=0,colors=heat.colors(100),
+                     shift=0,trans=I,by.resids=FALSE,scheme=0,hcolors=heat.colors(100),
                      contour.col=1,...) {
 ## plot method function for soap.smooth terms
   if (scheme==3) {  
@@ -760,7 +760,7 @@ plot.soap.film <- function(x,P=NULL,data=NULL,label="",se1.mult=1,se2.mult=2,
                      partial.resids=partial.resids,rug=rug,se=se,scale=scale,n=n,n2=n2,
                      pers=pers,theta=theta,phi=phi,jit=jit,xlab=xlab,ylab=ylab,main=main,
                      ylim=ylim,xlim=xlim,too.far=too.far,shade=shade,shade.col=shade.col,
-                     shift=shift,trans=trans,by.resids=by.resids,scheme=scheme,colors=colors,
+                     shift=shift,trans=trans,by.resids=by.resids,hcolors=hcolors, ## don't pass scheme!!
                      contour.col=contour.col,...)
             if (outline) { if (is.null(names(P$bnd))) {
                 for (i in 1:length(P$bnd)) lines(P$bnd[[i]],lwd=2)
@@ -789,7 +789,7 @@ plot.soap.film <- function(x,P=NULL,data=NULL,label="",se1.mult=1,se2.mult=2,
         ylim <- range(P$yscale);dy = ylim[2] - ylim[1]
         plot(P$xscale[1],P$yscale[1],xlab=P$xlab,ylab=P$ylab,main=P$main,xlim=xlim,ylim=ylim,...)
         rect(xlim[1]-dx,ylim[1]-dy,xlim[2]+dx,ylim[2]+dy,col="lightgrey")
-        image(P$xscale,P$yscale,P$fit,add=TRUE,col=colors,...)
+        image(P$xscale,P$yscale,P$fit,add=TRUE,col=hcolors,...)
         contour(P$xscale,P$yscale,P$fit,add=TRUE,col=contour.col,...)
       } else if (scheme==1) {
         image(P$xscale,P$yscale,P$fit,col=grey(0:50/50),xlab=P$xlab,
